#!/bin/sh
# install the system on hdd

set -e

while [ $# != 0 ]; do
  case $1 in
    -s) system_dev=$2
        shift 2
        ;;
    -d) data_dev=$2
        shift 2
        ;;
    --df) data_format=true
        shift 1
        ;;
    --dp) data_path=$2
        shift 2
        ;;
    -i) install_packages=$2
        shift 2
        ;;
    -p) post_install=1
        shift 1
        ;;
    -h) show_help=1
        break
        ;;
     *) show_help=1
        break
        ;;
  esac
done

if [ -z "${post_install}" -a -z "${system_dev}" -o "${show_help}" ]; then
  echo "Usage $0 {-p | -s DEVICE [-d DEVICE] [-i PACKAGES]}" >&2
  echo "  -s DEVICE     System device to install on" >&2
  echo "  -d DEVICE     Data device" >&2
  echo "  --df          Format data device" >&2
  echo "  --dp PATH     Path on data device" >&2
  echo "  -i PACKAGES   Packages to install" >&2
  echo "  -p            Do postinstall action" >&2
  exit 1
fi

export LC_ALL=C

if [ "${post_install}" ]; then
  packages=$(setting get install.packages)
  if [ "${packages}" ]; then
    export DEBIAN_FRONTEND=noninteractive
    setting set install.failed false
    apt-get --error-on=any update || setting set install.failed true
    apt-get install -q -y $( (echo ${packages} | tr " " $'\n'; apt-cache --generate pkgnames) | sort | uniq -d) || setting set install.failed true
  fi | while read line; do
    echo -e "$line\r"
    [ "$line" ] || continue
    [ "${line##Get*}" ] || ([ "${line##*Packages*}" -a "${line##*InRelease*}" ] && plymouth display-message --text="Downloading $(echo "$line" | cut -d " " -f4)" || plymouth display-message --text="Downloading updates")
    [ "${line##Unpacking*}" ] || plymouth display-message --text="$(echo "$line" | cut -d " " -f1-2)"
    [ "${line##Setting up*}" ] || plymouth display-message --text="$(echo "$line" | cut -d " " -f1-3)"
    [ "${line##Removing*}" ] || plymouth display-message --text="$(echo "$line" | cut -d " " -f1-2)"
  done

  if [ "$(setting get install.failed)" != "true" ]; then
    systemctl disable install
    sed "s/vga=0x314/vga=current/" -i /boot/syslinux/syslinux.cfg
    sed "s/vga=0x314/vga=current/" -i /boot/efi/boot/syslinux.cfg
  fi
  exit
fi


if [ ! -e "${system_dev}" ]; then
  echo "Unknown device '${system_dev}'" >&2
  exit 2
fi
if [ "${data_dev}" -a ! -e "${data_dev}" ]; then
  echo "Unknown device '${data_dev}'" >&2
  exit 2
fi


step=0
steps=8
progress()
{
  [ "$2" ] && steps=$((steps + $2))
  step=$((step + 1))
  echo "$((step * 100 / steps))% $1"
}


progress "Start installing on ${system_dev}"

# Unmount the target device
umount ${system_dev}* 2>/dev/null || true

boot_size=100

part_prefix=""
if [ ! "${system_dev#/dev/mmcblk}" = "${system_dev}" ] || \
   [ ! "${system_dev#/dev/nvme}" = "${system_dev}" ]; then
    part_prefix="p"
fi
boot_part=${system_dev}${part_prefix}1
root_part=${system_dev}${part_prefix}2
data_part=${data_dev}

progress "Store Package list..."
setting set install {failed:false}
if [ "${install_packages}" ]; then
  setting set install.packages "${install_packages}"
  systemctl enable install 2>/dev/null
fi


progress "Creating partition table..."
parted()
(
  set -o pipefail
  /usr/sbin/parted $* 2>&1 | (grep -v "^Information:\|^\s*$" || true) >&2
)
dd if=/dev/zero of=${system_dev} bs=512 count=35 2>/dev/null
parted ${system_dev} mktable gpt
parted ${system_dev} mkpart boot fat32 0% $boot_size
parted ${system_dev} set 1 legacy_boot on
parted ${system_dev} mkpart root btrfs $boot_size 100%
# create bios boot partition entry for rpi
[ -x /usr/sbin/gdisk ] && printf %s\\n r h 1 n c y n w y | gdisk ${system_dev} >/dev/null
sleep 1


progress "Formatting boot partition to fat..."
mkfs.vfat -F 32 ${boot_part} >/dev/null
progress "Formatting root partition to btrfs..."
mkfs.btrfs -q -f ${root_part}
if [ "${data_part}" -a "${data_format}" ]; then
  progress "Formatting data partition to xfs..." 1
  mkfs.xfs -q -f ${data_part}
fi


progress "Creating boot filesystem..."
boot_dev=/dev/$(mount | sed -n "s|^\(/dev/\|/run/media/\)\([^/ ]*\)[1-9/].* /rootfs.ro .*|\2|p")
[ -e ${boot_dev}1 ] && boot_dev=${boot_dev}1
boot_tgt=/tmp/boot_tgt
boot_src=/tmp/boot_src
mkdir -p ${boot_tgt}
mkdir -p ${boot_src}

umount ${boot_tgt} 2>/dev/null || true
umount ${boot_src} 2>/dev/null || true
mount ${boot_part} ${boot_tgt}
mount ${boot_dev} ${boot_src} -o ro 2>/dev/null || mount --bind /boot ${boot_src}

if [ -d ${boot_src}/isolinux ]; then
  cp ${boot_src}/{initrd,bzImage} ${boot_tgt}

  mkdir -p ${boot_tgt}/syslinux
  cp ${boot_src}/isolinux/splash.lss ${boot_tgt}/syslinux/
  cp /usr/share/syslinux/{ldlinux.c32,libcom32.c32,libutil.c32,vesamenu.c32} ${boot_tgt}/syslinux/

  mkdir -p ${boot_tgt}/efi/boot
  cp ${boot_src}/isolinux/splash.lss ${boot_tgt}/efi/boot/
  cp /usr/share/syslinux/efi64/{ldlinux.e64,libcom32.c32,libutil.c32,vesamenu.c32} ${boot_tgt}/efi/boot/
  cp /usr/share/syslinux/efi64/syslinux.efi ${boot_tgt}/efi/boot/bootx64.efi

  root_uuid=$(blkid ${root_part} | sed 's/.*UUID="\(\S*\)".*/\1/')
  sed "s/root=\S*/root=UUID=${root_uuid}/g" ${boot_src}/isolinux/isolinux.cfg > ${boot_tgt}/syslinux/syslinux.cfg
  sed "s/vga=current/vga=0x314/g" -i ${boot_tgt}/syslinux/syslinux.cfg
  cp ${boot_tgt}/syslinux/syslinux.cfg ${boot_tgt}/efi/boot/

  sync
  syslinux --install --directory syslinux ${boot_part} 2>/dev/null
  cat /usr/share/syslinux/gptmbr.bin > ${system_dev}
else
  cp -r ${boot_src}/* ${boot_tgt}

  if [ -e ${boot_tgt}/cmdline.txt ]; then
    root_uuid=$(blkid ${root_part} | sed 's/.*UUID="\(\S*\)".*/\1/')
    sed "s/root=.* rootwait /root=UUID=${root_uuid} /g" -i ${boot_tgt}/cmdline.txt
  fi
fi

umount ${boot_tgt}
umount ${boot_src}


if [ -d /sys/firmware/efi/efivars ] && efibootmgr &>/dev/null; then
  progress "Add uefi boot entry..." 1
  efibootmgr | grep " MLD$" | cut -b5-8 | while read n; do echo $n; efibootmgr -b $n -B; done >/dev/null
  efibootmgr -c -d ${system_dev} -L "MLD" -l "/efi/boot/bootx64.efi" > /dev/null
fi


progress "Creating root filesystem..."
root_tgt=/tmp/root_tgt
root_src=/tmp/root_src
mkdir -p ${root_tgt}
mkdir -p ${root_src}
mkdir -p /data

umount ${root_tgt} 2>/dev/null || true
umount ${root_src} 2>/dev/null || true
mount ${root_part} ${root_tgt} -o subvol=/
mount -t overlay overlay -o "ro,lowerdir=/rootfs.ro,upperdir=/rootfs.rw/upperdir,workdir=/rootfs.rw/work,noatime,nodiratime" ${root_src}

#btrfs subvolume create ${root_tgt}/@cache
btrfs subvolume create ${root_tgt}/@root >/dev/null
btrfs subvolume set-default $(btrfs subvolume list ${root_tgt} | grep " @root" | cut -d " " -f2) ${root_tgt}/@root

btrfs subvolume create ${root_tgt}/@data >/dev/null
root_size=$(/usr/sbin/parted -sm ${root_part} unit gib print | grep ^${root_part} | cut -d: -f2 | sed "s/\(\..*\)\?GiB//")
if [ ${root_size:-0} -ge 20 ]; then
  btrfs quota enable ${root_tgt}
  btrfs qgroup limit $((${root_size} - 10))G ${root_tgt}/@data
fi

cp -a ${root_src}/* ${root_tgt}/@root
rm -rf ${root_tgt}/@root/rootfs.r?

boot_uuid=$(blkid ${boot_part} | sed 's/.*UUID="\(\S*\)".*/\1/')
echo "UUID=${boot_uuid}  /boot  auto  defaults  1  2" >> ${root_tgt}/@root/etc/fstab
if [ "${data_part}" ]; then
  data_uuid=$(blkid ${data_part} | sed 's/.*UUID="\(\S*\)".*/\1/')
  echo "UUID=${data_uuid}  /mnt/${data_part##*/}  auto  defaults  1  2" >> ${root_tgt}/@root/etc/fstab
  echo "/mnt/${data_part##*/}${data_path:+/$data_path}  /data  none  bind  0  0" >> ${root_tgt}/@root/etc/fstab
else
  mkdir -p ${root_tgt}/@root/mnt/@data
  root_uuid=$(blkid ${root_part} | sed 's/.*UUID="\(\S*\)".*/\1/')
  echo "UUID=${root_uuid}  /mnt/${root_part##*/}  btrfs  subvol=@data  0  0" >> ${root_tgt}/@root/etc/fstab
  echo "/mnt/${root_part##*/}  /data  none  bind  0  0" >> ${root_tgt}/@root/etc/fstab
fi

# We dont want udev to mount our root and data device while we're booting...
echo "${system_dev}" >> ${root_tgt}/@root/etc/udev/mount.blacklist
[ "${data_dev}" ] && echo "${data_dev}" >> ${root_tgt}/@root/etc/udev/mount.blacklist

umount ${root_tgt}
umount ${root_src}


progress "Finish installing"
